/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package picto;

import java.io.File;
import javax.swing.UIManager;
import picto.app.gui.AboutWindow;
import picto.app.gui.MainWindow;
import picto.app.interfaces.Locales;
import picto.exceptions.IncorrectHeaderException;
import picto.utils.Config;
import picto.utils.Picto;
import picto.utils.PictoSolver;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class Main {

    private static String LAF = UIManager.getSystemLookAndFeelClassName();

    private Main() {
    }

    public static void main(String[] args) {
        if (args.length != 0) {
            evaluateArgs(args);
        }
        
        try {
            UIManager.setLookAndFeel(LAF);
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
        }
        
        new MainWindow().setVisible(true);
    }

    private static void evaluateArgs(String... args) {
        int i = 0;
        
        for (String arg : args) {
            
            if (arg.equalsIgnoreCase("-H")
                    || arg.equalsIgnoreCase("--HELP")) {
                System.out.println(getHelp());
                System.exit(0);
            } else if (arg.equalsIgnoreCase("-V")
                    || arg.equalsIgnoreCase("--VERSION")) {
                System.out.println("My Minesweeper Version " + Config.getAppVersion());
                System.exit(0);
            } else if (arg.equalsIgnoreCase("-L")
                    || arg.equalsIgnoreCase("--LICENCE")) {
                System.out.println(getLicense());
                System.exit(0);
            } else if (arg.equalsIgnoreCase("-C")
                    || arg.equalsIgnoreCase("--CREDITS")) {
                System.out.println(AboutWindow.loadCredits());
                System.exit(0);
            } else if (arg.equalsIgnoreCase("-W")
                    || arg.equalsIgnoreCase("--CHANGELOG")) {
                System.out.println(AboutWindow.loadChangeLog());
                System.exit(0);
            } else if (arg.equalsIgnoreCase("-O")
                    || arg.equalsIgnoreCase("--OPEN")) {
                System.out.println("Sorry not supported yet...");
                System.exit(0);
            } else if (arg.startsWith("-S=")
                    || arg.startsWith("--SOLVE=")) {
                String temp = arg.replace("-S=", "");
                temp = temp.replace("--SOLVE", "");
                solve(temp);
                System.exit(0);
            }
            
            boolean istheme = selectTheme(arg);
            boolean islocale = false;
            
            if (!istheme) {
                islocale = selectLocale(arg);
            }
            
            if (!istheme && !islocale) {
                System.out.format("Command \"%s\" not recognized! Try --HELP\n", arg);
                System.exit(0);
            }
            
            i++;
        }
    }

    public static Boolean selectTheme(String arg) {
        String temp = arg;
        
        if (arg.startsWith("-T=") || arg.startsWith("-t=")) {
            arg = arg.substring(3);
        } else if (arg.startsWith("--THEME=") || arg.startsWith("--theme=")) {
            arg = arg.substring(8);
        }
        
        if (temp.length() == arg.length()) {
            return false;
        }
        
        if (arg.equalsIgnoreCase("Java")) {
            LAF = UIManager.getLookAndFeel().toString();
            return true;
        } else if (arg.equalsIgnoreCase("Metal")) {
            LAF = UIManager.getCrossPlatformLookAndFeelClassName();
            return true;
        } else if (arg.equalsIgnoreCase("System")) {
            LAF = UIManager.getSystemLookAndFeelClassName();
            return true;
        } else if (arg.equalsIgnoreCase("GTK")) {
            LAF = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
            return true;
        } else if (arg.equalsIgnoreCase("Nimbus")) {
            LAF = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
            return true;
        } else {
            System.out.format("Theme \"%s\" not recognized! Try --HELP\n", arg);
            System.exit(0);
        }
        
        return false;
    }

    public static Boolean selectLocale(String arg) {
        String temp = arg;
        
        if (arg.startsWith("-L=") || arg.startsWith("-l=")) {
            arg = arg.substring(3);
        } else if (arg.startsWith("--LOCALE=") || arg.startsWith("--locale=")) {
            arg = arg.substring(9);
        }
        
        if (temp.length() == arg.length()) {
            return false;
        }
        
        if (arg.equalsIgnoreCase("EN")) {
            Config.setLocale(Locales.ENGLISH);
            return true;
        } else if (arg.equalsIgnoreCase("ES")) {
            Config.setLocale(Locales.SPANISH);
            return true;
        } else if (arg.equalsIgnoreCase("ZH")){
            Config.setLocale(Locales.SIMPLIFIED_CHINESE);
            return true;
        } else if (arg.equalsIgnoreCase("JP")){
            Config.setLocale(Locales.SIMPLIFIED_CHINESE);
            return true;
        }  else {
            System.out.format("Locale \"%s\" not recognized! Try --HELP\n", arg);
            System.exit(0);
        }
        
        return false;
    }

    /**
     * @return License string
     */
    private static String getLicense() {
        return "\n                  ..::PictoGame::..\n"
                + Config.getAppDescription()
                + "\n\n Copyright (C) 2010-2011 Federico Vera <dktcoding [at] gmail>\n\n"
                + "    This program is free software: you can redistribute it and/or modify\n"
                + "it under the terms of the GNU General Public License as published by\n"
                + "the Free Software Foundation, either version 3 of the License, or\n"
                + "(at your option) any later version.\n\n"
                + "    This program is distributed in the hope that it will be useful,\n"
                + "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
                + "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
                + "GNU General Public License for more details.\n\n"
                + "    You should have received a copy of the GNU General Public License\n"
                + "along with this program.  If not, see <http://www.gnu.org/licenses/>.\n";
    }

    /**
     * @return Help string
     */
    private static String getHelp() {
        return "\n\t\t\t" + Config.getAppName()
                + "\n\t" + Config.getAppDescription()
                + "\n\nUsage:\n"
                + "\tpicto [--OPTIONS]\n"
                + "\tpicto [--THEME=something][--LOCALE=locale]\n\n"
                + "Options:\n"
                + "\t-H,  --HELP              Shows this message\n"
                + "\t-V,  --VERSION           Shows version string\n"
                + "\t-W,  --CHANGELOG         Shows the changelog\n"
                + "\t-C,  --CREDITS           Shows credits\n"
                + "\t-L,  --LICENCE           Shows licence\n"
                + "\t-T=, --THEME=something   Starts the program with a different\n"
                + "\t                         look & feel, the available themes are:\n"
                + "\t                         Java, Metal, System, GTK, Nimbus\n"
                + "\t-L=, --LOCALE=locale     Starts the program with a different\n"
                + "\t                         locale; available languages are:\n"
                + "\t                         EN, SP, ZH, JP\n"
                + "\t-O=, --OPEN=\"PATH_TO_FILE/FILE\"\n\n"
                + "\t-S=, --SOLVE=\"PATH_TO_FILE/FILE\"\n\n"
                + "\tDefault excecution is: picto -T=System -L=EN";
    }
    
    private static void solve(String filename) {
        File file = new File(filename);
        
        if (!file.exists()){
            System.out.format("Couldn't find '%s'\n", filename);
            System.exit(0);
        }
        
        PictoSolver ps = new PictoSolver(new Picto(file));
        
        try {
            ps.solve();
            System.out.println(ps);
        } catch (IncorrectHeaderException ex) {
            System.out.println(ex.getMessage());
        }
        
    }
}
