/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import picto.app.interfaces.Focusable;
import picto.app.interfaces.Updatable;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class TextButton extends JButton implements Updatable, Focusable, ActionListener{
    private static final long serialVersionUID = 56456565324256L;
    private static Color headerBackgroundFocusColor = Config.getColor("headerBackgroundFocusColor");
    private static Color headerBackgroundColor = Config.getColor("headerBackgroundColor");
    private static Color strikedTextForeground = Config.getColor("strikedTextForeground");
    private static Color headerFontForeground = Config.getColor("headerFontForeground");
    private static Color headerBorderColor = Config.getColor("headerBorderColor");
    private static Font headerFont = Config.getFont("headerFont");
    private boolean pressed = false;
    private final Byte number;
    private boolean is_column;

    public TextButton(byte num, boolean is_column, byte i){
        super();
        this.is_column = is_column;
        this.number = num;
        setFocusable(false);
        setDoubleBuffered(false);
        setContentAreaFilled(false);
        setOpaque(true);
        setBackground(headerBackgroundColor);
        setBorder(i);
        if (number != 0){
            setForeground(headerFontForeground);
            setFont(Config.getFont("headerFont"));
            setText(number.toString());
            addActionListener(this);
        }
    }
    private static final String start = "<html><strike>";
    private static final String end = "</strike></html>";

    @Override 
    public void reset(){
        if (number != 0){
            setForeground(headerFontForeground);
            setText(number.toString());
        }
    }

    private byte top = 1, left = 1, bottom = 0, right = 0;
    private void setBorder(byte i){
        if (is_column){
            top = bottom = 0;
            if (i % 5 == 0){
                left = 1;
            } else if(i % 5 == 4) {
                right = 1;
            }
        } else {
            left = right = 0;
            if (i % 5 == 0){
                top = 1;
            } else if(i % 5 == 4) {
                bottom = 1;
            }
        }
        setBorder(BorderFactory.createMatteBorder(top, left, 
                                                  bottom, right, 
                                                  headerBorderColor));
    }

    @Override 
    public void giveFocus(){
        if (number != 0){
            setBackground(headerBackgroundFocusColor);
        }
    }

    @Override public void removeFocus(){
        if (number != 0){
            setBackground(headerBackgroundColor);
        }
    }

    @Override 
    public void setStatus(byte stat){
        if (stat == TEXT_CROSSED){
            pressed = true;
            setForeground(strikedTextForeground);
            setText(start+number+end);
        }
    }

    @Override 
    public byte getStatus(){
        return (number == 0) ? -1 : pressed ? TEXT_CROSSED : TEXT_NORMAL;
    }

    @Override 
    public void update() {
        headerBackgroundFocusColor = Config.getColor("headerBackgroundFocusColor");
        headerBackgroundColor = Config.getColor("headerBackgroundColor");
        strikedTextForeground = Config.getColor("strikedTextForeground");
        headerFontForeground = Config.getColor("headerFontForeground");
        headerBorderColor = Config.getColor("headerBorderColor");
        headerFont = Config.getFont("headerFont");
        setBackground(headerBackgroundColor);
        setBorder(BorderFactory.createMatteBorder(top, left, bottom, right
                        , headerBorderColor));
        if (number == 0){return;}
        if (getStatus() == TEXT_CROSSED){
            setForeground(strikedTextForeground);
            setFont(headerFont);
        } else {
            setForeground(headerFontForeground);
            setFont(headerFont);
        }
    }
    
    @Override 
    public void actionPerformed(ActionEvent e) {
        pressed = !pressed;
        if (pressed){
            setForeground(strikedTextForeground);
            setText(start+number+end);
        } else {
            reset();
        }
    }
}
